<%flags>
    # myghtyjax.myt needs to inherit from either None, or an autohandler
    # that doesnt have any textual output.
    inherit=None
    trim="both"
</%flags>

<%doc>
    init is called to register the names of javascript functions that will
    map to Myghty components and/or python function definitions.  supply
    key=value arguments, where keys are the names of javascript functions 
    to be generated and the value is one of: a function object, the string
    address of a Myghty component (path or method name), or a Myghty component object.
    This method should be called in the %init section of the calling component, usually before
    anything else has executed.  If it returns true it means the XMLHttpRequest action
    is taking effect, and the calling component should return immediately.  
    If false is returned, the component should continue on normally. 

    components that are called can specify options for how they should be handled via their
    <%attr> sections.  the available options are:

    type='source'
                    take the HTML source of the component and send it to a supplied javascript
                    callback function.

    type='writedom' 
                    take the HTML source of the component and send it to the DOM element
                    with the id given by either the <%attr> 'dom_id', or the first argument
                    to the javascript function.

    type='exec' 
                    take the output of the component and evaluate it directly as javascript.
</%doc>
<%method init>
    <%init>
        frame = m.execution_stack.pop()
        try:
            ret = init(m, **ARGS)
        finally:
            m.execution_stack.append(frame)
        return ret
    </%init>
</%method>

<%doc>
    js is called after init has been called, and delivers the javascript "stub" functions
    that connect page javascript to server side calls via XMLHttpRequest.
    
    the scripturi and handleruri arguments are optional arguments that 
    reference the web-accessible URIs of the myghtyjax.js and myghtyjax.myt 
    files respectively.  

    If these parameters are not supplied, they will be searched for in the interpreter 
    attributes "myghtyjax.handleruri" and "myghtyjax.scripturi".  
    if not there either, they will be determined based on the path of the original 
    calling component.
</%doc>
<%method js>
    <%args>
        scripturi = None
        handleruri = None
    </%args>
    <%init>
        js(m, handleruri = handleruri, scripturi = scripturi)
    </%init>
</%method>

<%args>
    _mjax_def
    _mjax_component
</%args>

<%init>
    m.comp(_mjax_component, **ARGS)
</%init>


<%once>

import inspect, string, types, posixpath
import myghty.request as request

# initialization method, receives javascript function names mapped
# to python function pointers, component names, or component objects.
# referenced by the "init" myghty method.
def init(m, **params):
    run_func = m.request_args.get('_mjax_def', None)
    
    if (run_func is not None):
        object = params[run_func]
        callable = _get_callable(m, run_func, object)
        callable.run(m)
        return True
    else:
        defs = m.attributes.setdefault('__mjax_defs', {})
        for (jsname, object) in params.iteritems():
                    defs[jsname] = _get_callable(m, jsname, object)
        return False
    
def _get_callable(m, name, object):
    if type(object) == types.FunctionType:    
        return DefCallable(name, object)
    else:
        return ComponentCallable(m, name, object)
        
def js(m, handleruri = None, scripturi = None):
    try:
        defs = m.attributes['__mjax_defs']
    except KeyError:
        raise "myghtyjax.js() method requires components and defs to be init()ialized first"

    path = m.current_component.path

    if handleruri is None:
        handleruri = m.interpreter.attributes.get('myghtyjax.handleruri', path)
        

    if scripturi is None:
        scripturi = m.interpreter.attributes.get('myghtyjax.scripturi', None)
        if scripturi is None:
            (dir, file) = posixpath.split(path)
            scripturi = posixpath.join(dir, "myghtyjax.js")
    
    m.write("<script src=\"%s\"></script>\n" % scripturi)
    m.write("<script>\n");
    for callable in defs.values():
        callable.callerpath = m.caller.path
        if callable.exectype == 'writedom':
            m.comp('SELF:write_writedom', callable = callable, handleruri = handleruri)
        elif callable.exectype == 'source':
            m.comp('SELF:write_docall', callable = callable, handleruri=handleruri)
        else:    
            m.comp('SELF:write_remotejs', callable = callable, handleruri=handleruri)

    m.write("</script>\n")

class JaxComponent:
    def get_function_args(self, leading_comma = False):
        args = string.join(self.argnames, ',')
        if leading_comma and args:
            return ', ' + args
        else:
            return args
        
    def get_remote_args(self, m):
        return string.join( 
                    [
                        "'_mjax_def' : '%s'" % self.jsname,
                        "'_mjax_component' : '%s'" % self.callerpath
                    ] + 
                    ["'%s' : %s" % (name, name) for name in self.argnames],
                    ",\n",
                )

class ComponentCallable(JaxComponent):
    def __init__(self, m, jsname, component):
        self.jsname = jsname
        self.component = component
    
        if type(component) == types.StringType:
            componentobj = m.fetch_component(component)
        else:
            componentobj = component

        self.exectype = componentobj.attributes.get('type', 'exec')
        self.dom_id = componentobj.attributes.get('dom_id', None)
    
        self.argnames = (componentobj.required_request_args + 
                    componentobj.request_args + 
                    componentobj.required_subrequest_args + 
                    componentobj.subrequest_args + 
                    componentobj.required_args + 
                    componentobj.args)
        
                
    def run(self, m):
        args = {}
        for name in self.argnames:
            if (m.root_request_args.has_key(name)):
                args[name] = m.root_request_args[name]
            elif (m.request_args.has_key(name)):
                args[name] = m.request_args[name]
                            
        m.comp(self.component, **args)
        
    
class DefCallable(JaxComponent):
    def __init__(self, jsname, defobj):
        self.jsname = jsname
        self.defobj = defobj
        self.argnames = inspect.getargspec(defobj)[0]
        self.exectype = 'exec'    
                
    def run(self, m):
        args = {}
        for name in self.argnames:
            args[name] = m.root_request_args[name]
            
        m.write(self.defobj(**args))

</%once>

<%method write_docall>
    <%args>
        callable
        handleruri
    </%args>
    function <% callable.jsname %>(callback<% callable.get_function_args(leading_comma = True) %>) {
        doCall('<% handleruri %>', callback, {
<% callable.get_remote_args(m) %>
        });
    }
</%method>

<%method write_remotejs>
    <%args>
        callable
        handleruri
    </%args>
    function <% callable.jsname %>(<% callable.get_function_args() %>) {
        runRemoteJS('<% handleruri %>', {
<% callable.get_remote_args(m) %>
        });
    }
</%method>

<%method write_writedom>
    <%args>
        callable
        handleruri
    </%args>
% if callable.dom_id is None:
    function <% callable.jsname %>(domid <% callable.get_function_args(leading_comma = True) %>) {
        populateDOM(domid, '<% handleruri %>', {
<% callable.get_remote_args(m) %>
        });
    }
% else:
    function <% callable.jsname %>(<% callable.get_function_args() %>) {
        populateDOM('<% callable.dom_id %>', '<% handleruri %>', {
<% callable.get_remote_args(m) %>
        });
    }
%
</%method>


